@@ -223,26 +223,32 @@ gda_connection_new (GdaClient *client,
 
 	/* retrieve correct username/password */
 	if (username)
-		real_username = username;
+		real_username = g_strdup (username);
 	else {
 		if (dsn_info->username)
-			real_username = dsn_info->username;
+			real_username = g_strdup (dsn_info->username);
 		else {
-			real_username = gda_quark_list_find (params, "USER");
-			if (real_username)
+			gchar *s;
+			s = gda_quark_list_find (params, "USER");
+			if (s) {
+				real_username = g_strdup (s);
 				gda_quark_list_remove (params, "USER");
+			}
 		}
 	}
 
 	if (password)
-		real_password = password;
+		real_password = g_strdup (password);
 	else {
 		if (dsn_info->password)
-			real_password = dsn_info->password;
+			real_password = g_strdup (dsn_info->password);
 		else {
-			real_password = gda_quark_list_find (params, "PASSWORD");
-			if (real_password)
+			gchar *s;
+			s = gda_quark_list_find (params, "PASSWORD");
+			if (s) {
+				real_password = g_strdup (s);
 				gda_quark_list_remove (params, "PASSWORD");
+			}
 		}
 	}
 
@@ -255,8 +261,8 @@ gda_connection_new (GdaClient *client,
 	cnc->priv->dsn = g_strdup (dsn);
 	cnc->priv->cnc_string = g_strdup (dsn_info->cnc_string);
 	cnc->priv->provider = g_strdup (dsn_info->provider);
-	cnc->priv->username = g_strdup (real_username);
-	cnc->priv->password = g_strdup (real_password);
+	cnc->priv->username = real_username;
+	cnc->priv->password = real_password;
 	cnc->priv->options = options;
 
 	gda_config_free_data_source_info (dsn_info);
@@ -771,6 +777,7 @@ gda_connection_begin_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 	gboolean retval;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
 	retval = gda_server_provider_begin_transaction (cnc->priv->provider_obj, cnc, xaction);
 	if (retval)
@@ -796,6 +803,7 @@ gda_connection_commit_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 	gboolean retval;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
 	retval = gda_server_provider_commit_transaction (cnc->priv->provider_obj, cnc, xaction);
 	if (retval)
@@ -822,6 +830,8 @@ gda_connection_rollback_transaction (GdaConnection *cnc, GdaTransaction *xaction
 	gboolean retval;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
+
 	retval = gda_server_provider_rollback_transaction (cnc->priv->provider_obj, cnc, xaction);
 	if (retval)
 		gda_client_notify_transaction_cancelled_event (cnc->priv->client, cnc, xaction);
@@ -890,3 +900,22 @@ gda_connection_get_errors (GdaConnection *cnc)
 	return cnc->priv->error_list;
 }
 
+/**
+ * gda_connection_create_blob
+ * @cnc: a #GdaConnection object.
+ * @blob: a user-allocated #GdaBlob structure.
+ *
+ * Creates a BLOB (Binary Large OBject) with read/write access.
+ *
+ * Returns: FALSE if the database does not support BLOBs. TRUE otherwise
+ * and the GdaBlob is created and ready to be used.
+ */
+gboolean
+gda_connection_create_blob (GdaConnection *cnc, GdaBlob *blob)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (blob != NULL, FALSE);
+
+	return gda_server_provider_create_blob (cnc->priv->provider_obj, cnc, blob);
+}
+
